<?php

namespace App\Helpers;

use Carbon\Carbon;

class Publics {

	static public function http_post( $name ) {
		return ( isset( $_POST ) and isset( $_POST[ $name ] ) and ! empty( $_POST[ $name ] ) ) ? $_POST[ $name ] : null;
	}

	static public function http_get( $name ) {
		return ( isset( $_GET ) and isset( $_GET[ $name ] ) and ! empty( $_GET[ $name ] ) ) ? $_GET[ $name ] : null;
	}

	static public function dump_r( $array ) {
		echo '<pre>';
		print_r( $array );
		echo '</pre>';
	}


	public static function fullDate( $value, $time = true ) {
		$_date = Carbon::parse( $value, TIME_ZONE );
		$date  = $_date->year . '/' . ( ( $_date->month < 10 ) ? '0' . $_date->month : $_date->month ) . '/' . ( ( $_date->day < 10 ) ? '0' . $_date->day : $_date->day ) . ' ' . $_date->hour . ':' . ( ( $_date->minute < 10 ) ? '0' . $_date->minute : $_date->minute ) . ':' . ( ( $_date->second < 10 ) ? '0' . $_date->second : $_date->second );
		if ( $time ) {
			return date( 'Y-m-d [ h:i A ]', strtotime( $date ) );
		}

		return date( 'Y-m-d', strtotime( $date ) );
	}

	public static function timeToHour( $time ) {
		return date( 'h:i A', $time );
	}

	public static function dateRTL( $value ) {
		$_date = Carbon::parse( $value, TIME_ZONE );
		$date  = $_date->year . '/' . ( ( $_date->month < 10 ) ? '0' . $_date->month : $_date->month ) . '/' . ( ( $_date->day < 10 ) ? '0' . $_date->day : $_date->day ) . ' ' . $_date->hour . ':' . ( ( $_date->minute < 10 ) ? '0' . $_date->minute : $_date->minute ) . ':' . ( ( $_date->second < 10 ) ? '0' . $_date->second : $_date->second );

		return date( 'Y-m-d', strtotime( $date ) );
	}

	static public function create_slug( $str ) {
		if ( $str !== mb_convert_encoding( mb_convert_encoding( $str, 'UTF-32', 'UTF-8' ), 'UTF-8', 'UTF-32' ) ) {
			$str = mb_convert_encoding( $str, 'UTF-8', mb_detect_encoding( $str ) );
		}
		$str = htmlentities( $str, ENT_NOQUOTES, 'UTF-8' );
		$str = preg_replace( '`&([a-z]{1,2})(acute|uml|circ|grave|ring|cedil|slash|tilde|caron|lig);`i', '\\1', $str );
		$str = html_entity_decode( $str, ENT_NOQUOTES, 'UTF-8' );
		$str = preg_replace( array( '`[^a-z0-9]`i', '`[-]+`' ), '-', $str );
		$str = strtolower( trim( $str, '-' ) );
		$str = substr( $str, 0, 100 );

		return $str;
	}

	static public function selected( $name, $value ) {
		if ( $name == $value ) {
			return 'selected="selected"';
		}
	}

	static public function ng_selected( $name, $value ) {
		if ( $name == $value ) {
			return 'ng-selected="selected"';
		}
	}

	static public function checked( $name, $value ) {
		if ( $name == $value ) {
			return 'checked="checked"';
		}
	}

	static function global_js( $name = null, $value = null ) {
		static $attached = false;
		static $variables = [];
		if ( ! $attached ) {
			$provide = function() use ( &$variables ) {
				if ( ! empty( $variables ) ) {
					echo( "<script type=\"text/javascript\">\n\n" );
					foreach ( $variables as $name => $value ) {
						echo( "    window['$name'] = JSON.parse('" . json_encode( $value ) . "');\n" );
						unset( $variables[ $name ] );
					}
					echo( "\n</script>\n" );
				}
			};

			add_action( 'wp_print_scripts', $provide, 0 );
			add_action( 'wp_print_footer_scripts', $provide, 0 );
			$attached = true;
		}
		if ( is_null( $name ) && is_null( $value ) ) {
			return $variables;
		}

		return $variables[ $name ] = $value;
	}

	static public function http_response_code( $code = null ) {
		if ( $code !== null ) {
			switch ( $code ) {
				case 100:
					$text = 'Continue';
					break;
				case 101:
					$text = 'Switching Protocols';
					break;
				case 200:
					$text = 'OK';
					break;
				case 201:
					$text = 'Created';
					break;
				case 202:
					$text = 'Accepted';
					break;
				case 203:
					$text = 'Non-Authoritative Information';
					break;
				case 204:
					$text = 'No Content';
					break;
				case 205:
					$text = 'Reset Content';
					break;
				case 206:
					$text = 'Partial Content';
					break;
				case 300:
					$text = 'Multiple Choices';
					break;
				case 301:
					$text = 'Moved Permanently';
					break;
				case 302:
					$text = 'Moved Temporarily';
					break;
				case 303:
					$text = 'See Other';
					break;
				case 304:
					$text = 'Not Modified';
					break;
				case 305:
					$text = 'Use Proxy';
					break;
				case 400:
					$text = 'Bad Request';
					break;
				case 401:
					$text = 'Unauthorized';
					break;
				case 402:
					$text = 'Payment Required';
					break;
				case 403:
					$text = 'Forbidden';
					break;
				case 404:
					$text = 'Not Found';
					break;
				case 405:
					$text = 'Method Not Allowed';
					break;
				case 406:
					$text = 'Not Acceptable';
					break;
				case 407:
					$text = 'Proxy Authentication Required';
					break;
				case 408:
					$text = 'Request Time-out';
					break;
				case 409:
					$text = 'Conflict';
					break;
				case 410:
					$text = 'Gone';
					break;
				case 411:
					$text = 'Length Required';
					break;
				case 412:
					$text = 'Precondition Failed';
					break;
				case 413:
					$text = 'Request Entity Too Large';
					break;
				case 414:
					$text = 'Request-URI Too Large';
					break;
				case 415:
					$text = 'Unsupported Media Type';
					break;
				case 422:
					$text = 'Unprocessable Entity Explained';
					break;
				case 500:
					$text = 'Internal Server Error';
					break;
				case 501:
					$text = 'Not Implemented';
					break;
				case 502:
					$text = 'Bad Gateway';
					break;
				case 503:
					$text = 'Service Unavailable';
					break;
				case 504:
					$text = 'Gateway Time-out';
					break;
				case 505:
					$text = 'HTTP Version not supported';
					break;
				default:
					exit( 'Unknown http status code "' . htmlentities( $code ) . '"' );
					break;
			}
			$protocol = ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' );
			header( $protocol . ' ' . $code . ' ' . $text );
			$GLOBALS['http_response_code'] = $code;
		} else {
			$code = ( isset( $GLOBALS['http_response_code'] ) ? $GLOBALS['http_response_code'] : 200 );
		}

		return $code;
	}

	static public function get_client_ip() {
		$ipaddress = '';
		if ( isset( $_SERVER['HTTP_CLIENT_IP'] ) ) {
			$ipaddress = $_SERVER['HTTP_CLIENT_IP'];
		} else if ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
			$ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else if ( isset( $_SERVER['HTTP_X_FORWARDED'] ) ) {
			$ipaddress = $_SERVER['HTTP_X_FORWARDED'];
		} else if ( isset( $_SERVER['HTTP_FORWARDED_FOR'] ) ) {
			$ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
		} else if ( isset( $_SERVER['HTTP_FORWARDED'] ) ) {
			$ipaddress = $_SERVER['HTTP_FORWARDED'];
		} else if ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
			$ipaddress = $_SERVER['REMOTE_ADDR'];
		} else {
			$ipaddress = 'UNKNOWN';
		}

		return $ipaddress;
	}

}
